@@ -24,7 +24,7 @@ from .conf.utils import BadLine, KeyAction, to_cmdline
 from .config import common_opts_as_dict, prepare_config_file_for_editing
 from .constants import (
     appname, config_dir, is_macos, is_wayland, kitty_exe,
-    supports_primary_selection
+    supports_primary_selection, website_url
 )
 from .fast_data_types import (
     CLOSE_BEING_CONFIRMED, IMPERATIVE_CLOSE_REQUESTED, NO_CLOSE_REQUESTED,
@@ -32,11 +32,12 @@ from .fast_data_types import (
     background_opacity_of, change_background_opacity, change_os_window_state,
     cocoa_set_menubar_title, create_os_window,
     current_application_quit_request, current_os_window, destroy_global_data,
-    focus_os_window, get_clipboard_string, get_options, global_font_size,
-    mark_os_window_for_close, os_window_font_size, patch_global_colors,
-    safe_pipe, set_application_quit_request, set_background_image, set_boss,
-    set_clipboard_string, set_in_sequence_mode, set_options, thread_write,
-    toggle_fullscreen, toggle_maximized
+    focus_os_window, get_clipboard_string, get_options, get_os_window_size,
+    global_font_size, mark_os_window_for_close, os_window_font_size,
+    patch_global_colors, safe_pipe, set_application_quit_request,
+    set_background_image, set_boss, set_clipboard_string, set_in_sequence_mode,
+    set_options, set_os_window_size, thread_write, toggle_fullscreen,
+    toggle_maximized
 )
 from .keys import get_shortcut, shortcut_matches
 from .layout.base import set_layout_options
@@ -52,10 +53,11 @@ from .tabs import (
 from .types import SingleKey
 from .typing import PopenType, TypedDict
 from .utils import (
-    func_name, get_editor, get_primary_selection, is_path_in_temp_dir,
-    log_error, open_url, parse_address_spec, parse_uri_list,
-    platform_window_id, read_shell_environment, remove_socket_file, safe_print,
-    set_primary_selection, single_instance, startup_notification_handler
+    func_name, get_editor, get_new_os_window_size, get_primary_selection,
+    is_path_in_temp_dir, log_error, open_url, parse_address_spec,
+    parse_uri_list, platform_window_id, read_shell_environment,
+    remove_socket_file, safe_print, set_primary_selection, single_instance,
+    startup_notification_handler
 )
 from .window import MatchPatternType, Window
 
@@ -332,6 +334,7 @@ class Boss:
         return self.add_os_window(startup_session)
 
     def new_os_window(self, *args: str) -> None:
+        '@ac:win: New OS Window'
         self._new_os_window(args)
 
     @property
@@ -341,6 +344,7 @@ class Boss:
             return t.active_window_for_cwd
 
     def new_os_window_with_cwd(self, *args: str) -> None:
+        '@ac:win: New OS Window with the same working directory as the currently active window'
         w = self.active_window_for_cwd
         cwd_from = w.child.pid_for_cwd if w is not None else None
         self._new_os_window(args, cwd_from)
@@ -377,6 +381,15 @@ class Boss:
         return response
 
     def remote_control(self, *args: str) -> None:
+        '''
+        @ac:misc: Run a remote control command
+
+        For example::
+
+            map F1 remote_control set-spacing margin=30
+
+        See :ref:`rc_mapping` for details.
+        '''
         from .rc.base import (
             PayloadGetter, command_for_name, parse_subcommand_cli
         )
@@ -489,6 +502,7 @@ class Boss:
             self.child_monitor.mark_for_close(window.id)
 
     def close_tab(self, tab: Optional[Tab] = None) -> None:
+        '@ac:tab: Close the current tab'
         tab = tab or self.active_tab
         if tab:
             self.confirm_tab_close(tab)
@@ -520,11 +534,13 @@ class Boss:
         for window in tab:
             self.close_window(window)
 
-    def toggle_fullscreen(self) -> None:
-        toggle_fullscreen()
+    def toggle_fullscreen(self, os_window_id: int = 0) -> None:
+        '@ac:win: Toggle the fullscreen status of the active OS Window'
+        toggle_fullscreen(os_window_id)
 
-    def toggle_maximized(self) -> None:
-        toggle_maximized()
+    def toggle_maximized(self, os_window_id: int = 0) -> None:
+        '@ac:win: Toggle the maximized status of the active OS Window'
+        toggle_maximized(os_window_id)
 
     def start(self, first_os_window_id: int) -> None:
         if not getattr(self, 'io_thread_started', False):
@@ -551,6 +567,21 @@ class Boss:
                 tm.resize()
 
     def clear_terminal(self, action: str, only_active: bool) -> None:
+        '''
+        @ac:misc: Clear the terminal
+
+        See :sc:`reset_terminal` for details. For example::
+
+            # Reset the terminal
+            map kitty_mod+f9 clear_terminal reset active
+            # Clear the terminal screen by erasing all contents
+            map kitty_mod+f10 clear_terminal clear active
+            # Clear the terminal scrollback by erasing it
+            map kitty_mod+f11 clear_terminal scrollback active
+            # Scroll the contents of the screen into the scrollback
+            map kitty_mod+f12 clear_terminal scroll active
+
+        '''
         if only_active:
             windows = []
             w = self.active_window
@@ -585,6 +616,11 @@ class Boss:
         self.change_font_size(True, None, new_size)
 
     def change_font_size(self, all_windows: bool, increment_operation: Optional[str], amt: float) -> None:
+        '''
+        @ac:win: Change the font size for the current or all OS Windows
+
+        See :ref:`conf-kitty-shortcuts.fonts` for details.
+        '''
         def calc_new_size(old_size: float) -> float:
             new_size = old_size
             if amt == 0:
@@ -641,6 +677,15 @@ class Boss:
         change_background_opacity(os_window_id, max(0.1, min(opacity, 1.0)))
 
     def set_background_opacity(self, opacity: str) -> None:
+        '''
+        @ac:win: Set the background opacity for the active OS Window
+
+        For example::
+
+            map f1 set_background_opacity +0.1
+            map f2 set_background_opacity -0.1
+            map f3 set_background_opacity 0.5
+        '''
         window = self.active_window
         if window is None or not opacity:
             return
@@ -717,6 +762,11 @@ class Boss:
                 self.dispatch_action(matched_action)
 
     def start_resizing_window(self) -> None:
+        '''
+        @ac:win: Resize the active window interactively
+
+        See :ref:`window_resizing` for details.
+        '''
         w = self.active_window
         if w is None:
             return
@@ -736,6 +786,16 @@ class Boss:
             return None
         return tab.resize_window_by(window.id, increment, is_horizontal)
 
+    def resize_os_window(self, os_window_id: int, width: int, height: int, unit: str, incremental: bool = False) -> None:
+        if not incremental and (width < 0 or height < 0):
+            return
+        metrics = get_os_window_size(os_window_id)
+        if metrics is None:
+            return
+        has_window_scaling = is_macos or is_wayland()
+        w, h = get_new_os_window_size(metrics, width, height, unit, incremental, has_window_scaling)
+        set_os_window_size(os_window_id, w, h)
+
     def default_bg_changed_for(self, window_id: int) -> None:
         w = self.window_id_map.get(window_id)
         if w is not None:
@@ -756,7 +816,8 @@ class Boss:
 
         def report_match(f: Callable) -> None:
             if self.args.debug_keyboard:
-                print(f'\x1b[35m{dispatch_type}\x1b[m matched action:', func_name(f), flush=True)
+                prefix = '\n' if dispatch_type == 'KeyPress' else ''
+                print(f'{prefix}\x1b[35m{dispatch_type}\x1b[m matched action:', func_name(f), flush=True)
 
         if key_action is not None:
             f = getattr(self, key_action.func, None)
@@ -783,6 +844,17 @@ class Boss:
         return False
 
     def combine(self, *actions: KeyAction) -> None:
+        '''
+        @ac:misc: Combine multiple actions and map to a single keypress
+
+        The syntax is::
+
+            map key combine <separator> action1 <separator> action2 <separator> action3 ...
+
+        For example::
+
+            map kitty_mod+e combine : new_window : next_layout
+        '''
         for key_action in actions:
             self.dispatch_action(key_action)
 
@@ -818,6 +890,7 @@ class Boss:
                 w.paste(text)
 
     def close_os_window(self) -> None:
+        '@ac:win: Close the currently active OS Window'
         tm = self.active_tab_manager
         if tm is not None:
             self.confirm_os_window_close(tm.os_window_id)
@@ -857,6 +930,7 @@ class Boss:
             action()
 
     def quit(self, *args: Any) -> None:
+        '@ac:win: Quit, closing all windows'
         tm = self.active_tab
         num = 0
         for q in self.os_window_map.values():
@@ -906,6 +980,8 @@ class Boss:
                     if exe:
                         cmd[0] = exe
 
+        if os.path.basename(cmd[0]) == 'less':
+            cmd.append('-+F')  # reset --quit-if-one-screen
         tab = self.active_tab
         if tab is not None:
             bdata = data.encode('utf-8') if isinstance(data, str) else data
@@ -915,6 +991,7 @@ class Boss:
                 )
 
     def edit_config_file(self, *a: Any) -> None:
+        '@ac:misc: Edit the kitty.conf config file in your favorite text editor'
         confpath = prepare_config_file_for_editing()
         # On macOS vim fails to handle SIGWINCH if it occurs early, so add a
         # small delay.
@@ -1005,6 +1082,7 @@ class Boss:
             return overlay_window
 
     def kitten(self, kitten: str, *args: str) -> None:
+        '@ac:misc: Run the specified kitten. See :doc:`/kittens/custom` for details'
         import shlex
         cmdline = args[0] if args else ''
         kargs = shlex.split(cmdline) if cmdline else []
@@ -1021,9 +1099,11 @@ class Boss:
             end_kitten(data, target_window_id, self)
 
     def input_unicode_character(self) -> None:
+        '@ac:misc: Input an arbitrary unicode character. See :doc:`/kittens/unicode-input` for details.'
         self._run_kitten('unicode_input')
 
     def set_tab_title(self) -> None:
+        '@ac:tab: Change the title of the active tab'
         tab = self.active_tab
         if tab:
             args = ['--name=tab-title', '--message', _('Enter the new title for this tab below.'), 'do_set_tab_title', str(tab.id)]
@@ -1042,6 +1122,7 @@ class Boss:
         self._run_kitten('show_error', args=['--title', title], input_data=msg)
 
     def create_marker(self) -> None:
+        '@ac:mk: Create a new marker'
         w = self.active_window
         if w:
             spec = None
@@ -1060,11 +1141,12 @@ class Boss:
 
             self._run_kitten('ask', [
                 '--name=create-marker', '--message',
-                _('Create marker, for example:\ntext 1 ERROR\nSee https://sw.kovidgoyal.net/kitty/marks.html\n')
+                _('Create marker, for example:\ntext 1 ERROR\nSee {}\n').format(website_url('marks'))
                 ],
                 custom_callback=done, action_on_removal=done2)
 
-    def kitty_shell(self, window_type: str) -> None:
+    def kitty_shell(self, window_type: str = 'window') -> None:
+        '@ac:misc: Run the kitty shell to control kitty with commands'
         kw: Dict[str, Any] = {}
         cmd = [kitty_exe(), '@']
         aw = self.active_window
@@ -1112,6 +1194,10 @@ class Boss:
         if not found_action:
             open_url(url, program or get_options().open_url_with, cwd=cwd)
 
+    def open_url_with_hints(self) -> None:
+        '@ac:misc: Click a URL using the keyboard'
+        self._run_kitten('hints')
+
     def drain_actions(self, actions: List) -> None:
 
         def callback(timer_id: Optional[int]) -> None:
@@ -1138,6 +1224,7 @@ class Boss:
                 w.paste(text)
 
     def paste_from_clipboard(self) -> None:
+        '@ac:cp: Paste from the clipboard to the active window'
         text = get_clipboard_string()
         self.paste_to_active_window(text)
 
@@ -1148,6 +1235,7 @@ class Boss:
         return get_primary_selection() if supports_primary_selection else get_clipboard_string()
 
     def paste_from_selection(self) -> None:
+        '@ac:cp: Paste from the clipboard to the active window'
         text = self.current_primary_selection_or_clipboard()
         self.paste_to_active_window(text)
 
@@ -1161,6 +1249,11 @@ class Boss:
                     self.copy_to_buffer(get_options().copy_on_select)
 
     def copy_to_buffer(self, buffer_name: str) -> None:
+        '''
+        @ac:cp: Copy the selection from the active window to the specified buffer
+
+        See :ref:`cpbuf` for details.
+        '''
         w = self.active_window
         if w is not None and not w.destroyed:
             text = w.text_for_selection()
@@ -1173,6 +1266,11 @@ class Boss:
                     self.clipboard_buffers[buffer_name] = text
 
     def paste_from_buffer(self, buffer_name: str) -> None:
+        '''
+        @ac:cp: Paste from the specified buffer to the active window
+
+        See :ref:`cpbuf` for details.
+        '''
         if buffer_name == 'clipboard':
             text: Optional[str] = get_clipboard_string()
         elif buffer_name == 'primary':
@@ -1183,6 +1281,11 @@ class Boss:
             self.paste_to_active_window(text)
 
     def goto_tab(self, tab_num: int) -> None:
+        '''
+        @ac:tab: Go to the specified tab, by number, starting with 1
+
+        Zero and negative numbers go to previously active tabs
+        '''
         tm = self.active_tab_manager
         if tm is not None:
             tm.goto_tab(tab_num - 1)
@@ -1194,11 +1297,13 @@ class Boss:
         return False
 
     def next_tab(self) -> None:
+        '@ac:tab: Make the next tab active'
         tm = self.active_tab_manager
         if tm is not None:
             tm.next_tab()
 
     def previous_tab(self) -> None:
+        '@ac:tab: Make the previous tab active'
         tm = self.active_tab_manager
         if tm is not None:
             tm.next_tab(-1)
@@ -1358,9 +1463,11 @@ class Boss:
         self._new_tab(args, as_neighbor=as_neighbor, cwd_from=cwd_from)
 
     def new_tab(self, *args: str) -> None:
+        '@ac:tab: Create a new tab'
         self._create_tab(list(args))
 
     def new_tab_with_cwd(self, *args: str) -> None:
+        '@ac:tab: Create a new tab with working directory for the window in it set to the same as the active window'
         w = self.active_window_for_cwd
         cwd_from = w.child.pid_for_cwd if w is not None else None
         self._create_tab(list(args), cwd_from=cwd_from)
@@ -1388,9 +1495,11 @@ class Boss:
                 return tab.new_window(cwd_from=cwd_from, location=location, allow_remote_control=allow_remote_control)
 
     def new_window(self, *args: str) -> None:
+        '@ac:win: Create a new window'
         self._new_window(list(args))
 
     def new_window_with_cwd(self, *args: str) -> None:
+        '@ac:win: Create a new window with working directory same as that of the active window'
         w = self.active_window_for_cwd
         if w is None:
             return self.new_window(*args)
@@ -1398,21 +1507,33 @@ class Boss:
         self._new_window(list(args), cwd_from=cwd_from)
 
     def launch(self, *args: str) -> None:
+        '''
+        @ac:misc: Launch the specified program in a new window/tab/etc.
+
+        See :doc:`launch` for details
+        '''
         from kitty.launch import launch, parse_launch_args
         opts, args_ = parse_launch_args(args)
         launch(self, opts, args_)
 
     def move_tab_forward(self) -> None:
+        '@ac:tab: Move the active tab forward'
         tm = self.active_tab_manager
         if tm is not None:
             tm.move_tab(1)
 
     def move_tab_backward(self) -> None:
+        '@ac:tab: Move the active tab backward'
         tm = self.active_tab_manager
         if tm is not None:
             tm.move_tab(-1)
 
     def disable_ligatures_in(self, where: Union[str, Iterable[Window]], strategy: int) -> None:
+        '''
+        @ac:misc: Turn on/off ligatures in the specified window
+
+        See :opt:`disable_ligatures` for details
+        '''
         if isinstance(where, str):
             windows: List[Window] = []
             if where == 'active':
@@ -1470,6 +1591,15 @@ class Boss:
             w.refresh()
 
     def load_config_file(self, *paths: str, apply_overrides: bool = True) -> None:
+        '''
+        @ac:misc: Reload the config file
+
+        If mapped without arguments reloads the default config file, otherwise loads
+        the specified config files, in order. Loading a config file *replaces* all
+        config options. For example::
+
+            map f5 load_config_file /path/to/some/kitty.conf
+        '''
         from .config import load_config
         old_opts = get_options()
         paths = paths or old_opts.config_paths
@@ -1527,6 +1657,13 @@ class Boss:
         self.show_error(_('Errors in kitty.conf'), msg)
 
     def set_colors(self, *args: str) -> None:
+        '''
+        @ac:misc: Change colors in the specified windows
+
+        For details, see :ref:`at_set-colors`. For example::
+
+            map f5 set_colors --configured /path/to/some/config/file/colors.conf
+        '''
         from kitty.rc.base import (
             PayloadGetter, command_for_name, parse_subcommand_cli
         )
@@ -1554,14 +1691,17 @@ class Boss:
             target_tab = tm.new_tab(empty_tab=True)
         else:
             target_os_window_id = target_os_window_id or current_os_window()
-            if target_tab_id == 'new':
+            if isinstance(target_tab_id, str):
                 if not isinstance(target_os_window_id, int):
                     q = self.active_tab_manager
                     assert q is not None
                     tm = q
                 else:
                     tm = self.os_window_map[target_os_window_id]
-                target_tab = tm.new_tab(empty_tab=True)
+                if target_tab_id == 'new':
+                    target_tab = tm.new_tab(empty_tab=True)
+                else:
+                    target_tab = tm.tab_at_location(target_tab_id) or tm.new_tab(empty_tab=True)
             else:
                 for tab in self.all_tabs:
                     if tab.id == target_tab_id:
@@ -1589,6 +1729,7 @@ class Boss:
         target_tab.make_active()
 
     def select_tab(self) -> None:
+        '@ac:tab: Interactively select a tab to switch to'
         title = 'Choose a tab to switch to'
         lines = [title, '']
         fmt = ': {1}'
@@ -1621,10 +1762,16 @@ class Boss:
         )
 
     def detach_window(self, *args: str) -> None:
+        '''
+        @ac:win: Detach a window, moving it to another tab or OS Window
+
+        See :ref:`detaching windows <detach_window>` for details.
+        '''
         if not args or args[0] == 'new':
             return self._move_window_to(target_os_window_id='new')
-        if args[0] == 'new-tab':
-            return self._move_window_to(target_tab_id='new')
+        if args[0] in ('new-tab', 'tab-prev', 'tab-left', 'tab-right'):
+            where = 'new' if args[0] == 'new-tab' else args[0][4:]
+            return self._move_window_to(target_tab_id=where)
         title = 'Choose a tab to move the window to'
         lines = [title, '']
         fmt = ': {1}'
@@ -1671,6 +1818,11 @@ class Boss:
         )
 
     def detach_tab(self, *args: str) -> None:
+        '''
+        @ac:tab: Detach a tab, moving it to another OS Window
+
+        See :ref:`detaching windows <detach_window>` for details.
+        '''
         if not args or args[0] == 'new':
             return self._move_tab_to()
 
@@ -1743,6 +1895,7 @@ class Boss:
             w.report_notification_activated(identifier)
 
     def show_kitty_env_vars(self) -> None:
+        '@ac:misc: Show the environment variables that the kitty process sees'
         w = self.active_window
         if w:
             output = '\n'.join(f'{k}={v}' for k, v in os.environ.items())
@@ -1767,6 +1920,7 @@ class Boss:
             tab.remove_window(w)
 
     def debug_config(self) -> None:
+        '@ac:misc: Show the effective configuration kitty is running with'
         from .debug_config import debug_config
         w = self.active_window
         if w is not None:
@@ -1774,3 +1928,8 @@ class Boss:
             set_clipboard_string(re.sub(r'\x1b.+?m', '', output))
             output += '\n\x1b[35mThis debug output has been copied to the clipboard\x1b[m'
             self.display_scrollback(w, output, title=_('Current kitty options'))
+
+    def discard_event(self) -> None:
+        '@ac:misc: Discard this event completely ignoring it'
+        pass
+    mouse_discard_event = discard_event
